import time

import cv2
import numpy as np
import socket


#INPUT SEND DATA
IP = '10.68.209.77' #paste your own IP here / 10.68.173.10 / 192.168.61.232
port = 6002
#----------------------------------------

# capture video
cap = cv2.VideoCapture(0, cv2.CAP_DSHOW)

#segment one color (or more)
#setting color limits (for red)

colorLowL = np.array([0, 100, 20], np.uint8)
colorUpL = np.array([10, 255, 255], np.uint8)

# define the kernel
kernel = np.ones((5, 5), "uint8")

# define the color name
cName = 'red'
cOut = (0, 0, 255)
run = True
minArea = 1000


centers =[]
detection = False

try:
    while (cap.isOpened()):
        ret, RGBframe = cap.read()

        #FIRST, COLOUR SEGMENTATION
        width = RGBframe.shape[1]
        height = RGBframe.shape[0]

        frame = (width, height)

        HSVframe = cv2.cvtColor(RGBframe, cv2.COLOR_BGR2HSV)

        # define the color mask
        colorMask = cv2.inRange(HSVframe, colorLowL, colorUpL)

        #
        colorMask = cv2.dilate(colorMask, kernel)
        colorRes = cv2.bitwise_and(RGBframe, RGBframe, mask=colorMask)

        #SECOND, GENERATE CONTOURS FOR THE SEGMENTED COLOUR ONLY
        gray = cv2.cvtColor(colorRes, cv2.COLOR_BGR2GRAY)
        blurred = cv2.GaussianBlur(gray, (5, 5), 0)
        thresh = cv2.threshold(gray, 0, 255, cv2.THRESH_BINARY)[1]

        contours, hierarchy = cv2.findContours(thresh, cv2.RETR_LIST, cv2.CHAIN_APPROX_SIMPLE)

        circleCent = []
        rectCent = []
        trgCent = []

        circles = []
        triangles = []
        rectangles = []

        cir_area = []
        rect_area = []
        tri_area = []

        areas = []

        #print(len(contours))
        #print(contours[1])
        M = cv2.moments(contours[0])
        cX = int(M["m10"] / M["m00"])
        cY = int(M["m01"] / M["m00"])
        #print(cX)

        items = []

        for c in contours:
            epsilon = 0.03 * cv2.arcLength(c, True)
            approx = cv2.approxPolyDP(c, epsilon, True)
            x = approx.ravel()[0]
            y = approx.ravel()[1] - 5

            area = cv2.contourArea(c)
            areas.append(area)

            if (area > minArea):

                items.append(c)

                if (10 > len(approx) > 5):
                    cir = cv2.putText(RGBframe, "circle", (x, y), cv2.FONT_HERSHEY_COMPLEX, 0.5, (0, 0, 0))
                    detection = True
                    cCent = [(cX, cY)]
                    circleCent.append(cCent)
                    circles.append(cir)

                    nC= cv2.contourArea(c)

                    cir_area.append(nC)


                if (len(approx) == 4):
                    rct = cv2.putText(RGBframe, "rectangle", (x, y), cv2.FONT_HERSHEY_COMPLEX, 0.5, (0, 0, 0))
                    detection = True

                    N = cv2.moments(c)
                    rX = int(N["m10"] / N["m00"])
                    rY = int(N["m01"] / N["m00"])

                    rCent = [(rX, rY)]
                    rectCent.append(rCent)
                    rectangles.append(rct)

                    rC = cv2.contourArea(c)
                    rect_area.append(rC)

                if (len(approx) == 3):
                    trg = cv2.putText(RGBframe, "triangle", (x, y), cv2.FONT_HERSHEY_COMPLEX, 0.5, (0, 0, 0))
                    detection = 'triangle:True'
                    U = cv2.moments(c)
                    tX = int(U["m10"] / U["m00"])
                    tY = int(U["m01"] / U["m00"])

                    tCent = [(tX, tY)]
                    trgCent.append(tCent)
                    triangles.append(trg)

                    tC = cv2.contourArea(c)
                    tri_area.append(tC)

            cv2.drawContours(RGBframe, [approx], 0, (0, 0, 0), 1)
            #print(len(approx))

        cv2.imshow("contour approximation", RGBframe)
        if cv2.waitKey(10) & 0xFF == ord('q'):
            break

        # LIVE DATA STREAMING MODULE

        n = 'shape detection (color segmented)' #input script title here
        d = detection
        c = 'circles.{}/triangles.{}/rectangles.{}'.format(circleCent, trgCent, rectCent)
        e = contours
        a = 'circles.{}/triangles.{}/rectangles.{}'.format(cir_area, tri_area, rect_area)
        s = 'circles.{}/triangles.{}/rectangles.{}'.format(len(circles), len(triangles), len(rectangles))
        i = len(items)
        f = frame
        template = str(
            'name:{}|detection:{}|center:{}|edge:{}|area:{}|shape:{}|items:{}|frame:{}'.format(n, d, c, e, a, s, i, f))
        IP = IP
        port = port

        print(len(contours))


        def UDP_client(IP, port, message):
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            sock.sendto(bytes(message, "utf-8"), (IP, port))


        UDP_client(IP, port, template)

    cap.release()
    cv2.destroyAllWindows()


except Exception as e:
    print(e)

